@@ -1,4 +1,5 @@
 from django.conf import settings
+from django.db import transaction
 from .models import Message
 from rapidsms.models import Backend, Connection
 from rapidsms.apps.base import AppBase
@@ -12,12 +13,6 @@ from urllib2 import urlopen
 import time
 import re
 
-# we keep a list of all the messages currently being processed here
-outgoing_pk_queue = []
-
-# our lock for that queue
-outgoing_queue_lock = Lock()
-
 # we make sure only one thread is modifying db messages a time
 outgoing_db_lock = Lock()
 
@@ -50,26 +45,25 @@ class HttpRouterThread(Thread, LoggerMixin):
         return self._isbusy
 
     def run(self):
-        global outgoing_pk_queue
-        global outgoing_queue_lock
-        global outgoing_db_lock
         global sending_mass_messages
-        
+
         while self.is_alive():
             if not sending_mass_messages:
-                outgoing_queue_lock.acquire()
+                transaction.enter_transaction_management()
 
                 try:
-                    # this gets any outgoing messages which are either pending or queued, exlcuding those
-                    # which are already being processed
-                    to_process = Message.objects.filter(direction='O',
-                                                        status__in=['P','Q']).order_by('status').exclude(pk__in=outgoing_pk_queue)
+                    # this gets any outgoing messages which are either pending or queued
+                    to_process = list(Message.objects.filter(direction='O',
+                                                             status__in=['P','Q']).order_by('status').for_single_update())
 
-                    if to_process.count():
+                    if len(to_process):
                         self._isbusy = True
                         outgoing_message = to_process[0]
-                        outgoing_pk_queue.append(outgoing_message.pk)
-                        outgoing_queue_lock.release()
+                        outgoing_message.status = 'L'
+                        outgoing_message.save()
+
+                        # frees our update lock
+                        transaction.commit()                        
 
                         # process the outgoing phases for this message
                         send_msg = get_router().process_outgoing_phases(outgoing_message)
@@ -77,37 +71,39 @@ class HttpRouterThread(Thread, LoggerMixin):
                         # if it wasn't cancelled, send it off
                         if send_msg:
                             self.send_message(outgoing_message)
-
-                        outgoing_queue_lock.acquire()
-                        outgoing_pk_queue.remove(outgoing_message.pk)
-                        outgoing_queue_lock.release()
                 except:
                     import traceback
                     traceback.print_exc()
-                        
+
                 finally:
                     try:
-                        outgoing_queue_lock.release()
+                        transaction.commit()
                     except:
-                        # it's ok, it wasn't locked
                         pass
+
             self._isbusy = False
             time.sleep(0.5)
 
-    def send_message(self, msg, **kwargs):
+    def fetch_url(self, url):
         """
-        Sends the message off.  We first try to directly contact our sms router to deliver it, 
-        if we fail, then we just add it to our outgoing queue.
+        Wrapper around url open, mostly here so we can monkey patch over it in unit tests.
         """
-        global outgoing_db_lock
+        response = urlopen(url, timeout=15)
+        return response.getcode()
+
+    def build_send_url(self, msg, kwargs):
+        """
+        Constructs an appropriate send url for the given message.
+        """
+        # first build up our list of parameters
         params = {
-            'backend': msg.connection.backend,
+            'backend': msg.connection.backend.name,
             'recipient': msg.connection.identity,
             'text': msg.text,
             'id': msg.pk
         }
 
-        # add any other backend-specific parameters from kwargs
+        # make sure our parameters are URL encoded
         params.update(kwargs)
         for k, v in params.items():
             try:
@@ -115,19 +111,52 @@ class HttpRouterThread(Thread, LoggerMixin):
             except UnicodeEncodeError:
                 params[k] = quote_plus(str(v.encode('UTF-8')))
 
+        # get our router URL
+        router_url = settings.ROUTER_URL
+
+        # is this actually a dict?  if so, we want to look up the appropriate backend
+        if type(router_url) is dict:
+            router_dict = router_url
+            backend_name = msg.connection.backend.name
+            
+            # is there an entry for this backend?
+            if backend_name in router_dict:
+                router_url = router_dict[backend_name]
+
+            # if not, look for a default backend 
+            elif 'default' in router_dict:
+                router_url = router_dict['default']
+
+            # none?  blow the hell up
+            else:
+                self.error("No router url mapping found for backend '%s', check your settings.ROUTER_URL setting" % backend_name)
+                raise Exception("No router url mapping found for backend '%s', check your settings.ROUTER_URL setting" % backend_name)
+
+        # return our built up url with all our variables substituted in
+        full_url = router_url % params
+
+        return full_url
+
+    def send_message(self, msg, **kwargs):
+        """
+        Sends the message off.  We first try to directly contact our sms router to deliver it, 
+        if we fail, then we just add it to our outgoing queue.
+        """
+        global outgoing_db_lock
+
         # and actually hand the message off to our router URL
         try:
-            # FIXME: clean this up
-            response = urlopen(settings.ROUTER_URL % params)
+            url = self.build_send_url(msg, kwargs)
+            status_code = self.fetch_url(url)
             outgoing_db_lock.acquire()
             # kannel likes to send 202 responses, really any
             # 2xx value means things went okay
-            if int(response.getcode()/100) == 2:
+            if int(status_code/100) == 2:
                 self.info("SMS[%d] SENT" % msg.id)
                 msg.status = 'S'
                 msg.save()
             else:
-                self.error("SMS[%d] Message not sent, got status: %s .. queued for later delivery." % (msg.id, response.getcode()))
+                self.error("SMS[%d] Message not sent, got status: %s .. queued for later delivery." % (msg.id, status_code))
                 msg.status = 'Q'
                 msg.save()
             outgoing_db_lock.release()
@@ -182,12 +211,13 @@ class HttpRouter(object, LoggerMixin):
 
         # finally, create our db message
         outgoing_db_lock.acquire()
+
         message = Message.objects.create(connection=connection,
                                          text=text,
                                          direction=direction,
                                          status=status)
         outgoing_db_lock.release()
-        
+
         return message
 
 
@@ -322,7 +352,7 @@ class HttpRouter(object, LoggerMixin):
                 if not worker.is_busy():
                     all_busy = False
                     break
-            
+
             if all_busy and len(outgoing_worker_threads) < num_workers:
                 worker = HttpRouterThread()
                 worker.daemon = True # they don't need to quit gracefully
@@ -397,9 +427,13 @@ class HttpRouter(object, LoggerMixin):
         try:
             cls = AppBase.find(module_name)
         except:
+            import traceback
+            traceback.print_exc()
             cls = None
 
-        if cls is None: return None
+        if cls is None:
+            self.error("Unable to find SMS application with module: '%s'" % module_name)
+            return None
 
         app = cls(self)
         self.apps.append(app)
